@@ -117,10 +117,10 @@ impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterConst<'a, T, N> {
         0 => None,
         _ => {
           let res = Some(&*self.start);
-          self.start = match size_of::<T>() {
-            0 => (self.start as usize + 1) as *const T,
-            _ => self.start.offset(1),
-          };
+          match size_of::<T>() {
+            0 => self.end = (self.end as *const u8).wrapping_offset(-1) as *const T,
+            _ => self.start = self.start.offset(1),
+          }
           res
         }
       }
@@ -146,9 +146,9 @@ impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterConst<'a, T, N> {
       unsafe {
         match size_of::<T>() {
           0 => {
-            let res = (self.start as usize + n) as *const T;
-            self.start = (res as usize + 1) as *const T;
-            Some(&*res)
+            self.end = (self.end as *const u8).wrapping_offset(-(n as isize)) as *const T;
+            self.start = (self.start as usize + 1) as *const T;
+            Some(&*self.start)
           }
           _ => {
             let res = self.start.add(n);
@@ -312,10 +312,10 @@ impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterMut<'a, T, N> {
         0 => None,
         _ => {
           let res = Some(&mut *self.start);
-          self.start = match size_of::<T>() {
-            0 => (self.start as usize + 1) as *mut T,
-            _ => self.start.offset(1),
-          };
+          match size_of::<T>() {
+            0 => self.end = (self.end as *mut u8).wrapping_offset(-1) as *mut T,
+            _ => self.start = self.start.offset(1),
+          }
           res
         }
       }
@@ -341,9 +341,9 @@ impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterMut<'a, T, N> {
       unsafe {
         match size_of::<T>() {
           0 => {
-            let res = (self.start as usize + n) as *mut T;
-            self.start = (res as usize + 1) as *mut T;
-            Some(&mut *res)
+            self.end = (self.end as *mut u8).wrapping_offset(-(n as isize)) as *mut T;
+            self.start = (self.start as usize + 1) as *mut T;
+            Some(&mut *self.start)
           }
           _ => {
             let res = self.start.add(n);
