@@ -29,7 +29,7 @@ use alloc::string::String;
 use alloc::vec::Vec;
 
 #[cfg(feature = "std")]
-use std::io::{self, BufRead, IoSlice, IoSliceMut, Read, ReadBuf, Write};
+use std::io::{self, BufRead, IoSlice, IoSliceMut, Read, ReadBuf};
 
 #[cfg(feature = "serde")]
 use core::marker::PhantomData;
@@ -726,25 +726,25 @@ impl<'a, T: 'a, const N: usize> const IntoIterator for &'a mut StaticVec<T, N> {
   }
 }
 
-impl<T, const N: usize> IntoIterator for StaticVec<T, N> {
+impl<T, const N: usize> const IntoIterator for StaticVec<T, N> {
   type IntoIter = StaticVecIntoIter<T, N>;
   type Item = T;
   /// Returns a by-value [`StaticVecIntoIter`](crate::iterators::StaticVecIntoIter) over the
   /// StaticVec's inhabited area, which consumes the StaticVec.
   #[inline(always)]
-  fn into_iter(mut self) -> Self::IntoIter {
+  fn into_iter(self) -> Self::IntoIter {
     let old_length = self.length;
-    // This prevents the values from being dropped locally, since they're
-    // being copied into the iterator.
-    self.length = 0;
     StaticVecIntoIter {
       start: 0,
       end: old_length,
       data: {
-        // Copy the inhabited part of self into the iterator.
+        // Copy the inhabited part of `self` into the iterator.
         let mut data = Self::new_data_uninit();
         unsafe {
-          self
+          // The `MaybeUninit` wrapping prevents the values from being dropped locally, which
+          // is necessary since again they're being copied into the iterator.
+          MaybeUninit::new(self)
+            .assume_init_ref()
             .as_ptr()
             .copy_to_nonoverlapping(Self::first_ptr_mut(&mut data), old_length)
         };
@@ -893,16 +893,35 @@ impl<const N: usize> Read for StaticVec<u8, N> {
   fn read_buf(&mut self, buf: &mut ReadBuf<'_>) -> io::Result<()> {
     // Here we directly adapt the implementation from &[u8].
     let amount = const_min(buf.remaining(), self.len());
-    let (a, b) = self.as_mut().split_at(amount);
-    buf.append(a);
-    *self = Self::from(b);
+    let b = self.split_off(amount);
+    buf.append(&self);
+    *self = b;
+    Ok(())
+  }
+}
+
+impl<const N: usize> fmt::Write for StaticVec<u8, N> {
+  #[inline]
+  fn write_str(&mut self, s: &str) -> fmt::Result {
+    // This is just exactly `try_extend_from_slice`, except with the specific `Result` type
+    // that this particular trait method calls for.
+    let old_length = self.length;
+    let added_length = s.len();
+    if N - old_length < added_length {
+      return Err(fmt::Error);
+    }
+    unsafe {
+      s.as_ptr()
+        .copy_to_nonoverlapping(self.mut_ptr_at_unchecked(old_length), added_length);
+      self.set_len(old_length + added_length);
+    }
     Ok(())
   }
 }
 
 #[cfg(feature = "std")]
 #[doc(cfg(feature = "std"))]
-impl<const N: usize> Write for StaticVec<u8, N> {
+impl<const N: usize> io::Write for StaticVec<u8, N> {
   #[inline]
   fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
     let old_length = self.length;
